from zope import schema
from zope.interface import implementer
from zope.interface import directlyProvides
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.annotation import interfaces as annot_ifaces
from zope import globalrequest

from plone import api
from plone.autoform.directives import order_before, omitted, widget

from collective.z3cform.datagridfield import DataGridFieldFactory, DictRow

from Products.Poi.content.tracker import IBasicData
from Products.Poi.content.tracker import ITracker as ITrackerOrig
from Products.Poi.content.tracker import Tracker as TrackerOrig

from sunpower.poi import _


def memoize_context_request(func):
    """
    Cache the function return per context and request.
    """
    memo_key = 'sunpower.memoize:{0}.{1}'.format(
        func.__module__, func.__name__)

    def memo_getter(context):
        """
        Lookup the value from the cache if available.
        """

        # Look up the value in the cache and return if present
        cache = None
        request = globalrequest.getRequest()
        if request is not None:
            annotations = annot_ifaces.IAnnotations(request)
            cache = annotations.get(memo_key)
            if cache is None:
                cache = annotations[memo_key] = {}
            cache_key = context.getPhysicalPath()
            if cache_key in cache:
                return cache[cache_key]

        value = func(context)

        # Store the value in the cache
        if cache is not None:
            cache[cache_key] = value

        return value

    return memo_getter


@memoize_context_request
def possibleOrigins(context):
    """
    Get the available origins as a Vocabulary.
    """
    if ISunPowerTracker.providedBy(context):
        tracker = context
    elif hasattr(context, 'getTracker'):
        tracker = context.getTracker()
    else:
        return SimpleVocabulary.fromValues([])
    terms = [
        SimpleTerm(value=tt.get('short_name'), title=tt.get('title'))
        for tt in tracker.available_origins
        if tt.get('short_name') is not None
    ]
    return SimpleVocabulary(terms)


@memoize_context_request
def possibleTypesOfIssues(context):
    """
    Get the available types of issues as a Vocabulary.
    """
    if ISunPowerTracker.providedBy(context):
        tracker = context
    elif hasattr(context, 'getTracker'):
        tracker = context.getTracker()
    else:
        return SimpleVocabulary.fromValues([])
    terms = [
        SimpleTerm(value=tt.get('short_name'), title=tt.get('title'))
        for tt in tracker.available_types_of_issues
        if tt.get('short_name') is not None
    ]
    return SimpleVocabulary(terms)


@memoize_context_request
def possibleClaimStatuses(context):
    """
    Get the available claim statuses as a Vocabulary.
    """
    if ISunPowerTracker.providedBy(context):
        tracker = context
    elif hasattr(context, 'getTracker'):
        tracker = context.getTracker()
    else:
        return SimpleVocabulary.fromValues([])
    terms = [
        SimpleTerm(value=tt.get('short_name'), title=tt.get('title'))
        for tt in tracker.available_claim_statuses
        if tt.get('short_name') is not None
    ]
    return SimpleVocabulary(terms)


@memoize_context_request
def possibleSubCategories(context):
    """
    Get the available sub-categories as a Vocabulary.
    """
    if ISunPowerTracker.providedBy(context):
        tracker = context
    elif hasattr(context, 'getTracker'):
        tracker = context.getTracker()
    else:
        return SimpleVocabulary.fromValues([])
    terms = [
        SimpleTerm(value=tt.get('short_name'), title=tt.get('title'))
        for tt in tracker.available_sub_categories
        if tt.get('short_name') is not None
    ]
    return SimpleVocabulary(terms)


@memoize_context_request
def possibleWhy1Options(context):
    """
    Get the available why 1 options as a Vocabulary.
    """
    if ISunPowerTracker.providedBy(context):
        tracker = context
    elif hasattr(context, 'getTracker'):
        tracker = context.getTracker()
    else:
        return SimpleVocabulary.fromValues([])
    terms = [
        SimpleTerm(value=tt.get('short_name'), title=tt.get('title'))
        for tt in tracker.available_why1
        if tt.get('short_name') is not None
    ]
    return SimpleVocabulary(terms)


@memoize_context_request
def possibleWhy2Options(context):
    """
    Get the available why 1 options as a Vocabulary.
    """
    if ISunPowerTracker.providedBy(context):
        tracker = context
    elif hasattr(context, 'getTracker'):
        tracker = context.getTracker()
    else:
        return SimpleVocabulary.fromValues([])
    terms = [
        SimpleTerm(value=tt.get('short_name'), title=tt.get('title'))
        for tt in tracker.available_why2
        if tt.get('short_name') is not None
    ]
    return SimpleVocabulary(terms)


@memoize_context_request
def possibleCarriers(context):
    """
    Get the available carriers as a Vocabulary.
    """
    if ISunPowerTracker.providedBy(context):
        tracker = context
    elif hasattr(context, 'getTracker'):
        tracker = context.getTracker()
    else:
        return SimpleVocabulary.fromValues([])
    terms = [
        SimpleTerm(value=tt.get('short_name'), title=tt.get('title'))
        for tt in tracker.available_carriers
        if tt.get('short_name') is not None
    ]
    return SimpleVocabulary(terms)


directlyProvides(possibleOrigins, IContextSourceBinder)
directlyProvides(possibleCarriers, IContextSourceBinder)
directlyProvides(possibleTypesOfIssues, IContextSourceBinder)
directlyProvides(possibleClaimStatuses, IContextSourceBinder)
directlyProvides(possibleSubCategories, IContextSourceBinder)
directlyProvides(possibleWhy1Options, IContextSourceBinder)
directlyProvides(possibleWhy2Options, IContextSourceBinder)


class ISunPowerTracker(ITrackerOrig):
    widget(available_origins=DataGridFieldFactory)
    available_origins = schema.List(
        title=_(u'Poi_label_availableOrigins',
                default=u"Origin Names"),
        default=[
            {'short_name': 'empty0',
             'title': 'No value',
             'description': 'Short name must be empty0 for validation to work'},
            {'short_name': 'C21',
             'title': 'C21',
             'description': ''},
            {'short_name': 'C21-Rialto',
             'title': 'C21-Rialto',
             'description': ''},
            {'short_name': 'Cranbury W04',
             'title': 'Cranbury W04',
             'description': ''},
            {'short_name': 'Cranbury',
             'title': 'Cranbury',
             'description': ''},
            {'short_name': 'Hayward',
             'title': 'Hayward',
             'description': ''},
            {'short_name': 'K+N Cranbury',
             'title': 'K+N Cranbury',
             'description': ''},
            {'short_name': 'K+N Hayward Depot',
             'title': 'K+N Hayward Depot',
             'description': ''},
            {'short_name': 'KN Rialto Warehouse',
             'title': 'KN Rialto Warehouse',
             'description': ''},
            {'short_name': 'KN Rialto WH',
             'title': 'KN Rialto WH',
             'description': ''},
            {'short_name': 'KN Rialto',
             'title': 'KN Rialto',
             'description': ''},
            {'short_name': 'KN WH',
             'title': 'KN WH',
             'description': ''},
            {'short_name': 'Kuehne-Nagel DC / Cranbury-KN10A and KN36E',
             'title': 'Kuehne-Nagel DC / Cranbury-KN10A and KN36E',
             'description': ''},
            {'short_name': 'Kuehne-Nagel DC / Hayward-KN48A',
             'title': 'Kuehne-Nagel DC / Hayward-KN48A',
             'description': ''},
            {'short_name': 'N/A',
             'title': 'N/A',
             'description': ''},
            {'short_name': 'RDC - Rialto',
             'title': 'RDC - Rialto',
             'description': ''},
            {'short_name': 'Rialto',
             'title': 'Rialto',
             'description': ''},
            {'short_name': 'Rialto-KN36E',
             'title': 'Rialto-KN36E',
             'description': ''},
            {'short_name': 'SunPower Kuehne+Nagel Warehouse Cranbury W04',
             'title': 'SunPower Kuehne+Nagel Warehouse Cranbury W04',
             'description': ''},
            {'short_name': 'SunPower Kuehne+Nagel Warehouse',
             'title': 'SunPower Kuehne+Nagel Warehouse',
             'description': ''},
            {'short_name': 'Sunpower',
             'title': 'Sunpower',
             'description': ''},
            {'short_name': 'W02 Hayward',
             'title': 'W02 Hayward',
             'description': ''},
            {'short_name': 'W02',
             'title': 'W02',
             'description': ''},
            {'short_name': 'W04',
             'title': 'W04',
             'description': ''},
            {'short_name': 'W04-Cranbury',
             'title': 'W04-Cranbury',
             'description': ''},
        ],
        value_type=DictRow(title=_(u'Origin Name'), schema=IBasicData),
    )

    widget(available_carriers=DataGridFieldFactory)
    available_carriers = schema.List(
        title=_(u'Poi_label_availableCarriers',
                default=u"Available Carriers"),
        value_type=DictRow(title=_(u'Carrier'), schema=IBasicData),
        required=False
    )

    widget(available_types_of_issues=DataGridFieldFactory)
    available_types_of_issues = schema.List(
        title=_(u'Poi_label_available_types_of_issues',
                default=u"Available Categories"),
        default=[
            {'short_name': 'empty0',
             'title': 'No value',
             'description': 'Short name must be empty0 for validation to work'},
            {'short_name': 'Damage',
             'title': 'Damage',
             'description': ''},
            {'short_name': 'DriverAssistEquip',
             'title': 'Driver Assist / Equip',
             'description': ''},
            {'short_name': 'IncompleteDelivery',
             'title': 'Incomplete Delivery',
             'description': ''},
            {'short_name': 'MiscComplaint',
             'title': 'Misc Complaint',
             'description': ''},
            {'short_name': 'OTDAppt',
             'title': 'OTD-Appt',
             'description': ''},
            {'short_name': 'OTDDay',
             'title': 'OTD-Day',
             'description': ''},
            {'short_name': 'PreCall',
             'title': 'Pre-Call',
             'description': ''},
        ],
        value_type=DictRow(title=_(u'Category'), schema=IBasicData),
    )

    widget(available_claim_statuses=DataGridFieldFactory)
    available_claim_statuses = schema.List(
        title=_(u'Poi_label_available_claim_statuses',
                default=u"Available Claim Statuses"),
        default=[
            {'short_name': 'NotAssessed',
             'title': 'Not assessed',
             'description': ''},
            {'short_name': 'NotRequired',
             'title': 'Not required',
             'description': ''},
            {'short_name': 'Required',
             'title': 'Required',
             'description': ''},
            {'short_name': 'Ready',
             'title': 'Ready',
             'description': ''},
            {'short_name': 'Processed',
             'title': 'Processed',
             'description': ''},
            {'short_name': 'Rejected',
             'title': 'Rejected',
             'description': ''},
        ],
        value_type=DictRow(title=_(u'Claim Status'), schema=IBasicData),
    )

    widget(available_sub_categories=DataGridFieldFactory)
    available_sub_categories = schema.List(
        title=_(u'Poi_label_available_sub_categories',
                default=u"Available Sub-Categories"),
        default=[
            {'short_name': 'empty0',
             'title': 'No value',
             'description': 'Short name must be empty0 for validation to work'},
            {'short_name': 'DamageToProductConcealed',
             'title': 'Damage to Product Concealed',
             'description': ''},
            {'short_name': 'DamageToProductVisible',
             'title': 'Damage to Product Visible',
             'description': ''},
            {'short_name': 'DamageToProperty',
             'title': 'Damage to Property',
             'description': ''},
            {'short_name': 'LateDelivery',
             'title': 'Late Delivery',
             'description': ''},
            {'short_name': 'MissedPickup',
             'title': 'Missed Pickup',
             'description': ''},
            {'short_name': 'MissingPallet',
             'title': 'Missing Pallet',
             'description': ''},
            {'short_name': 'MissingPaperwork',
             'title': 'Missing Paperwork',
             'description': ''},
            {'short_name': 'MissingQuantity',
             'title': 'Missing Quantity',
             'description': ''},
            {'short_name': 'ExcessQuantity',
             'title': 'Excess Quantity',
             'description': ''},
            {'short_name': 'WrongProduct',
             'title': 'Wrong Product',
             'description': ''},
            {'short_name': 'NoPreCall',
             'title': 'No Pre-Call',
             'description': ''},
            {'short_name': 'NoDriverAssist',
             'title': 'No Driver Assist',
             'description': ''},
            {'short_name': 'MissingPalletJack',
             'title': 'Missing Pallet Jack',
             'description': ''},
            {'short_name': 'MissingLiftGate',
             'title': 'Missing Lift Gate',
             'description': ''},
            {'short_name': 'LiftGateWeightExceeded',
             'title': 'Lift Gate Weight Exceeded',
             'description': ''},
        ],
        value_type=DictRow(title=_(u'Sub-Category'), schema=IBasicData),
    )

    widget(available_why1=DataGridFieldFactory)
    available_why1 = schema.List(
        title=_(u'Poi_label_available_why1',
                default=u"Available Why 1 Options"),
        default=[
            {'short_name': 'K-KN Warehouse',
             'title': 'K - KN Warehouse',
             'description': ''},
            {'short_name': 'M-Mechanical',
             'title': 'M - Mechanical',
             'description': ''},
            {'short_name': 'P-Planning',
             'title': 'P - Planning',
             'description': ''},
            {'short_name': 'R-ReTrans',
             'title': 'R - ReTrans',
             'description': ''},
            {'short_name': 'S-Shipper',
             'title': 'S - Shipper',
             'description': ''},
            {'short_name': 'V-Vendor',
             'title': 'V - Vendor',
             'description': ''},
            {'short_name': 'W-Weather',
             'title': 'W - Weather',
             'description': ''},
        ],
        value_type=DictRow(title=_(u'Why 1 Option'), schema=IBasicData),
    )

    widget(available_why2=DataGridFieldFactory)
    available_why2 = schema.List(
        title=_(u'Poi_label_available_why2',
                default=u"Available Why 2 Options"),
        default=[
            {'short_name': 'ChangesOutsideOfLeadTime',
             'title': 'Changes Outside of Lead Time',
             'description': ''},
            {'short_name': 'ForceMajeureNonWeather',
             'title': 'Force Majeure (Non-Weather)',
             'description': ''},
            {'short_name': 'FreightRouting',
             'title': 'Freight Routing',
             'description': ''},
            {'short_name': 'ImproperHandling',
             'title': 'Improper Handling',
             'description': ''},
            {'short_name': 'InboundErrorOrDelay',
             'title': 'Inbound Error or Delay',
             'description': ''},
            {'short_name': 'IncorrectInformation',
             'title': 'Incorrect Information',
             'description': ''},
            {'short_name': 'InsufficientPacking',
             'title': 'Insufficient Packing',
             'description': ''},
            {'short_name': 'InventoryDiscrepancy',
             'title': 'Inventory Discrepancy',
             'description': ''},
            {'short_name': 'LateOrderDrop',
             'title': 'Late Order Drop',
             'description': ''},
            {'short_name': 'LoadDidNotFitInTrailer',
             'title': 'Load did not fit in trailer',
             'description': ''},
            {'short_name': 'Mechanical',
             'title': 'Mechanical',
             'description': ''},
            {'short_name': 'MissingContact',
             'title': 'Missing Contact',
             'description': ''},
            {'short_name': 'PalletConsolidation',
             'title': 'Pallet Consolidation',
             'description': ''},
            {'short_name': 'PickError',
             'title': 'Pick Error',
             'description': ''},
            {'short_name': 'ShippingError',
             'title': 'Shipping Error',
             'description': ''},
            {'short_name': 'System-Ceil',
             'title': 'System - Ceil',
             'description': ''},
            {'short_name': 'System-EDI',
             'title': 'System - EDI',
             'description': ''},
            {'short_name': 'System-Oracle',
             'title': 'System - Oracle',
             'description': ''},
            {'short_name': 'System-TMS',
             'title': 'System - TMS',
             'description': ''},
            {'short_name': 'Weather',
             'title': 'Weather',
             'description': ''},
        ],
        value_type=DictRow(title=_(u'Why 2 Option'), schema=IBasicData),
    )

    # remove unused fields
    omitted('available_areas',
            'available_issue_types',
            'available_severities',
            'default_severity',
            'available_releases',
            )

    order_before(available_origins='assignees')
    order_before(available_carriers='assignees')
    order_before(available_types_of_issues='assignees')
    order_before(available_claim_statuses='assignees')
    order_before(available_sub_categories='assignees')
    order_before(available_why1='assignees')
    order_before(available_why2='assignees')


@implementer(ISunPowerTracker)
class SunPowerTracker(TrackerOrig):
    """
    A SunPower Poi Tracker
    """
    def getIssues(self):
        """get issues belonging to this tracker"""
        return api.content.find(
            portal_type='SunPowerIssue',
            context=self,
        )

    def getIssueWorkflowStates(self):
        """Get a DisplayList of the workflow states available on issues."""
        portal_workflow = api.portal.get_tool(name='portal_workflow')
        chain = portal_workflow.getChainForPortalType('SunPowerIssue')
        workflow = getattr(portal_workflow, chain[0])
        states = getattr(workflow, 'states')
        vocab = []
        for id, state in states.items():
            vocab.append((id, state.title))
        return vocab
